import 'package:article_flutter_riverpod/example.dart';
import 'package:article_flutter_riverpod/screens/example_group_screen.dart';
import 'package:article_flutter_riverpod/screens/example_screen.dart';
import 'package:flutter/material.dart';
import 'package:go_router/go_router.dart';

final GlobalKey<NavigatorState> _rootNavigatorKey = GlobalKey<NavigatorState>(debugLabel: 'root');
final GlobalKey<NavigatorState> _shellNavigatorKey = GlobalKey<NavigatorState>(debugLabel: 'shell');

class App extends StatelessWidget {
  App(
    this.groups, {
    Key? key,
  })  : router = buildRouter(groups),
        super(key: key);

  final List<ExampleGroup> groups;
  final GoRouter router;

  @override
  Widget build(BuildContext context) {
    return MaterialApp.router(
      title: 'Flutter par la pratique',
      theme: ThemeData(
        primarySwatch: Colors.blue,
      ),
      routeInformationProvider: router.routeInformationProvider,
      routerDelegate: router.routerDelegate,
      routeInformationParser: router.routeInformationParser,
    );
  }

  static GoRouter buildRouter(List<ExampleGroup> groups) {
    return GoRouter(
      navigatorKey: _rootNavigatorKey,
      initialLocation: groups[0].path,
      routes: [
        ShellRoute(
          navigatorKey: _shellNavigatorKey,
          builder: (_, __, child) => ScaffoldWithNavBar(groups, child: child),
          routes: [
            for (ExampleGroup group in groups) //
              GoRoute(
                path: group.path,
                builder: (_, __) => ExampleGroupScreen(group),
                routes: [
                  for (Example example in group.examples) //
                    GoRoute(
                      path: example.path,
                      builder: (_, __) => ExampleScreen(example),
                      parentNavigatorKey: _rootNavigatorKey,
                    ),
                ],
              ),
          ],
        )
      ],
    );
  }
}

class ScaffoldWithNavBar extends StatelessWidget {
  const ScaffoldWithNavBar(
    this.groups, {
    required this.child,
    Key? key,
  }) : super(key: key);

  final List<ExampleGroup> groups;
  final Widget child;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: child,
      bottomNavigationBar: BottomNavigationBar(
        items: [
          for (ExampleGroup group in groups) //
            BottomNavigationBarItem(
              icon: Icon(group.icon),
              label: group.title,
            ),
        ],
        currentIndex: _calculateSelectedIndex(context),
        onTap: (index) => _onItemTapped(index, context),
      ),
    );
  }

  int _calculateSelectedIndex(BuildContext context) {
    final GoRouter route = GoRouter.of(context);
    final String location = route.location;

    for (var i = 0; i < groups.length; i++) {
      if (location.startsWith(groups[i].path)) {
        return i;
      }
    }

    return 0;
  }

  void _onItemTapped(int index, BuildContext context) {
    context.go(groups[index].path);
  }
}
